@@ -803,7 +803,8 @@ svc_start(const char *svcName, int req)
 
 
 /**
- * Migrate a service to another node.
+ * Migrate a service to another node.  Relies on agent
+ * operating synchronously
  */
 int
 svc_migrate(const char *svcName, int target)
@@ -824,6 +825,11 @@ svc_migrate(const char *svcName, int target)
 		return RG_EINVAL;
 	}
 
+	if (m->cn_member == 0) {
+		free_member_list(membership);
+		return RG_ENODE;
+	}
+
 	count_resource_groups_local(m);
 	if (m->cn_svcexcl ||
 	    (m->cn_svccount && is_exclusive(svcName))) {
@@ -875,21 +881,8 @@ svc_migrate(const char *svcName, int target)
 		rg_unlock(&lockp);
 		return RG_EFROZEN;
 	}
-
-	/* LOCK HELD */
-	svcStatus.rs_owner = target;
-	svcStatus.rs_last_owner = my_id();
-	svcStatus.rs_state = RG_STATE_MIGRATE;
-	svcStatus.rs_transition = (uint64_t)time(NULL);
-
-	if (set_rg_state(svcName, &svcStatus) != 0) {
-		logt_print(LOG_ERR,
-		       "#75: Failed changing service status\n");
-		rg_unlock(&lockp);
-		return RG_EFAIL;
-	}
 	rg_unlock(&lockp);
-       
+
 	ret = group_migrate(svcName, target);
 
 	switch(ret) {
@@ -902,51 +895,34 @@ svc_migrate(const char *svcName, int target)
 		   the target node simply died; in this case, set status
 		   back to started */
 		return RG_EFAIL;
-		break;
 	case OCF_RA_NOT_RUNNING:
 		/* For these two, the VM was either not running or 
 		   migration is simply impossible. */
 		/* Don't mark the service as failed; since it's either
 		   recoverable or still running. */
-		ret = RG_EFAIL;
-		break;
+		return RG_EFAIL;
 	case OCF_RA_NOT_CONFIGURED:
-		ret = RG_EINVAL;
-		break;
+		return RG_EINVAL;
 	case 0:
-		return 0;
+		break;
 	}
 
-	/* Ok, we've hit a recoverable condition.  Since VMs and migratory
-	   services are ... well, migratable, we can just flip the state
-	   back to 'started' and error checking will fix it later. */
-	if (rg_lock(svcName, &lockp) < 0) {
-		logt_print(LOG_ERR, "#45: Unable to obtain cluster lock: %s\n",
-		       strerror(errno));
-		return ret;
+	/* Success - flip owner in state info */
+  	if (rg_lock(svcName, &lockp) < 0) {
+ 		logt_print(LOG_ERR, "#45b: Unable to obtain cluster lock: %s\n",
+			   strerror(errno));
+		return RG_EFAIL;
 	}
-
-	if (get_rg_state(svcName, &svcStatus) != 0) {
-		rg_unlock(&lockp);
-		logt_print(LOG_ERR, "#46: Failed getting status for RG %s\n",
-		       svcName);
-		return ret;
-	}
-
-	if (svcStatus.rs_last_owner != (uint32_t)my_id() ||
-	    svcStatus.rs_owner != (uint32_t)target ||
-	    svcStatus.rs_state != RG_STATE_MIGRATE) {
-		rg_unlock(&lockp);
-		return ret;
-	}
-
-	svcStatus.rs_owner = my_id();
+	
+	/* No need for a 'get' here since the service is still STARTED */
+	svcStatus.rs_last_owner = svcStatus.rs_owner;
+	svcStatus.rs_owner = target;
 	svcStatus.rs_state = RG_STATE_STARTED;
-
+  
 	set_rg_state(svcName, &svcStatus);
 	rg_unlock(&lockp);
-
-	return ret;
+  
+ 	return 0;
 }
 
 
@@ -1669,14 +1645,14 @@ svc_start_remote(const char *svcName, int request, uint32_t target)
  * @param new_owner	Member who actually ends up owning the service.
  */
 int
-handle_relocate_req(char *svcName, int request, int preferred_target,
+handle_relocate_req(char *svcName, int orig_request, int preferred_target,
 		    int *new_owner)
 {
 	cluster_member_list_t *allowed_nodes = NULL, *backup = NULL;
 	cman_node_t *m;
-	int target = preferred_target, me = my_id();
-	int ret, x;
 	rg_state_t svcStatus;
+	int target = preferred_target, me = my_id();
+	int ret, x, request = orig_request;
 	
 	get_rg_state_local(svcName, &svcStatus);
 	if (svcStatus.rs_state == RG_STATE_DISABLED ||
@@ -1778,6 +1754,13 @@ handle_relocate_req(char *svcName, int request, int preferred_target,
 				 */
 				return 0;
 			}
+
+			/*
+			 * Failed to start on that node.
+			 * Use the START_RECOVER operation on subsequent
+			 * attempts.
+			 */
+			request = RG_START_RECOVER;
 		}
 	}
 
@@ -1812,6 +1795,10 @@ handle_relocate_req(char *svcName, int request, int preferred_target,
 			return 0;
 		case RG_EDEPEND:
 		case RG_EFAIL:
+			/* Uh oh - we failed to relocate to this node.
+			   ensure that we tell the next node to start it from
+			   the 'recovering' state. */
+			request = RG_START_RECOVER;
 			memb_mark_down(allowed_nodes, target);
 			continue;
 		case RG_EABORT:
@@ -1844,7 +1831,7 @@ handle_relocate_req(char *svcName, int request, int preferred_target,
 	 * We got sent here from handle_start_req.
 	 * We're DONE.
 	 */
-	if (request == RG_START_RECOVER) {
+	if (orig_request == RG_START_RECOVER) {
 		_svc_stop_finish(svcName, 0, RG_STATE_STOPPED);
 		return RG_EFAIL;
 	}
@@ -2070,7 +2057,8 @@ handle_start_remote_req(char *svcName, int req)
 	if (need_check)
 		pthread_mutex_unlock(&exclusive_mutex);
 
-	if (svc_stop(svcName, RG_STOP_RECOVER) == 0)
+	if (svc_stop(svcName, central_events_enabled() ?
+		     RG_STATE_STOPPED : RG_STOP_RECOVER) == 0)
 		return RG_EFAIL;
 
 	svc_fail(svcName);
